@@ -37,10 +37,12 @@ Create an activity::
 
 """
 
-from threading import Thread
+import backoff
+import boto.exception as boto_exception
 import boto.swf.layer2 as swf
 import itertools
 import json
+import threading
 
 from garcon import log
 from garcon import utils
@@ -225,7 +227,7 @@ class ActivityInstance:
         try:
             for requirement in self.runner.requirements(self.global_context):
                 value = self.global_context.get(requirement)
-                if value:
+                if value is not None:
                     activity_input.update({requirement: value})
 
             activity_input.update({
@@ -245,15 +247,60 @@ class Activity(swf.ActivityWorker, log.GarconLogger):
     version = '1.0'
     task_list = None
 
-    def run(self):
+    @backoff.on_exception(
+        backoff.expo,
+        boto_exception.SWFResponseError,
+        max_tries=5,
+        giveup=utils.non_throttle_error,
+        on_backoff=utils.throttle_backoff_handler,
+        jitter=backoff.full_jitter)
+    def poll_for_activity(self, identity=None):
+        """Runs Activity Poll.
+
+        If a SWF throttling exception is raised during a poll, the poll will
+        be retried up to 5 times using exponential backoff algorithm.
+
+        Upgrading to boto3 would make this retry logic redundant.
+
+        Args:
+            identity (str): Identity of the worker making the request, which
+                is recorded in the ActivityTaskStarted event in the AWS
+                console. This enables diagnostic tracing when problems arise.
+        """
+
+        return self.poll(identity=identity)
+
+    def run(self, identity=None):
         """Activity Runner.
 
         Information is being pulled down from SWF and it checks if the Activity
         can be ran. As part of the information provided, the input of the
         previous activity is consumed (context).
+
+        Args:
+            activity_id (str): Identity of the worker making the request, which
+                is recorded in the ActivityTaskStarted event in the AWS
+                console. This enables diagnostic tracing when problems arise.
         """
 
-        activity_task = self.poll()
+        try:
+            if identity:
+                self.logger.debug('Polling with {}'.format(identity))
+            activity_task = self.poll_for_activity(identity)
+        except Exception as error:
+            # Catch exceptions raised during poll() to avoid an Activity thread
+            # dying & worker daemon unable to process the affected Activity.
+            # AWS api limits on SWF calls are a common source of such
+            # exceptions (see https://github.com/xethorn/garcon/pull/75)
+
+            # on_exception() can be overriden by the flow to send an alert
+            # when an exception occurs.
+            if self.on_exception:
+                self.on_exception(self, error)
+
+            self.logger.error(error, exc_info=True)
+            return True
+
         packed_context = activity_task.get('input')
         context = dict()
 
@@ -273,8 +320,9 @@ class Activity(swf.ActivityWorker, log.GarconLogger):
                     self.fail(reason=str(error)[:255])
                     if self.on_exception:
                         self.on_exception(self, error)
-                except:
-                    pass
+                except Exception as error2:  # noqa: E722
+                    if self.on_exception:
+                        self.on_exception(self, error2)
 
         self.unset_log_context()
         return True
@@ -423,7 +471,7 @@ class ActivityWorker():
             if (self.worker_activities and
                     activity.name not in self.worker_activities):
                 continue
-            Thread(target=worker_runner, args=(activity,)).start()
+            threading.Thread(target=worker_runner, args=(activity,)).start()
 
 
 class ActivityState:
