@@ -15,9 +15,10 @@ import json
 
 from garcon import activity
 from garcon import event
+from garcon import log
 
 
-class DeciderWorker(swf.Decider):
+class DeciderWorker(swf.Decider, log.GarconLogger):
 
     def __init__(self, flow, register=True):
         """Initialize the Decider Worker.
@@ -137,6 +138,7 @@ class DeciderWorker(swf.Decider):
             decisions.fail_workflow_execution(reason=str(e))
             if self.on_exception:
                 self.on_exception(self, e)
+            self.logger.error(e, exc_info=True)
 
     def delegate_decisions(self, decisions, decider, history, context):
         """Delegate the decisions.
@@ -177,8 +179,9 @@ class DeciderWorker(swf.Decider):
             decisions.fail_workflow_execution(reason=str(e))
             if self.on_exception:
                 self.on_exception(self, e)
+            self.logger.error(e, exc_info=True)
 
-    def run(self):
+    def run(self, identity=None):
         """Run the decider.
 
         The decider defines which task needs to be launched and when based on
@@ -191,12 +194,31 @@ class DeciderWorker(swf.Decider):
         If the decider is not able to find an uncompleted activity, the
         workflow can safely mark its execution as complete.
 
+        Args:
+            identity (str): Identity of the worker making the request, which
+                is recorded in the DecisionTaskStarted event in the AWS
+                console. This enables diagnostic tracing when problems arise.
+
         Return:
             boolean: Always return true, so any loop on run can act as a long
                 running process.
         """
 
-        poll = self.poll()
+        try:
+            poll = self.poll(identity=identity)
+        except Exception as error:
+            # Catch exceptions raised during poll() to avoid a Decider thread
+            # dying & the daemon unable to process subsequent workflows.
+            # AWS api limits on SWF calls are a common source of such
+            # exceptions.
+
+            # on_exception() can be overriden by the flow to send an alert
+            # when such an exception occurs.
+            if self.on_exception:
+                self.on_exception(self, error)
+            self.logger.error(error, exc_info=True)
+            return True
+
         custom_decider = getattr(self.flow, 'decider', None)
 
         if 'events' not in poll:
@@ -223,7 +245,7 @@ class ScheduleContext:
     Schedule Context
     ================
 
-    The schedule context keeps track of all the current scheduling progress – 
+    The schedule context keeps track of all the current scheduling progress –
     which allows to easy determinate if there are more decisions to be taken
     or if the execution can be closed.
     """
