@@ -1,10 +1,7 @@
-from __future__ import absolute_import
-from __future__ import print_function
-try:
-    from unittest.mock import MagicMock
-except:
-    from mock import MagicMock
+from unittest.mock import MagicMock
 import json
+
+import boto.exception as boto_exception
 import pytest
 
 from garcon import activity
@@ -62,6 +59,72 @@ def poll():
     return dict(activityId='something')
 
 
+def test_poll_for_activity(monkeypatch, poll=poll):
+    """Test that poll_for_activity successfully polls.
+    """
+
+    current_activity = activity_run(monkeypatch, poll)
+    current_activity.poll.return_value = 'foo'
+
+    activity_results = current_activity.poll_for_activity()
+    assert current_activity.poll.called
+    assert activity_results == 'foo'
+
+
+def test_poll_for_activity_throttle_retry(monkeypatch, poll=poll):
+    """Test that SWF throttles are retried during polling.
+    """
+
+    current_activity = activity_run(monkeypatch, poll)
+
+    response_status = 400
+    response_reason = 'Bad Request'
+    reponse_body = (
+        '{"__type": "com.amazon.coral.availability#ThrottlingException",'
+        '"message": "Rate exceeded"}')
+    json_body = json.loads(reponse_body)
+    exception = boto_exception.SWFResponseError(
+        response_status, response_reason, body=json_body)
+    current_activity.poll.side_effect = exception
+
+    with pytest.raises(boto_exception.SWFResponseError):
+        current_activity.poll_for_activity()
+    assert current_activity.poll.call_count == 5
+
+
+def test_poll_for_activity_error(monkeypatch, poll=poll):
+    """Test that non-throttle errors during poll are thrown.
+    """
+
+    current_activity = activity_run(monkeypatch, poll)
+
+    exception = Exception()
+    current_activity.poll.side_effect = exception
+
+    with pytest.raises(Exception):
+        current_activity.poll_for_activity()
+
+
+def test_poll_for_activity_identity(monkeypatch, poll=poll):
+    """Test that identity is passed to poll_for_activity.
+    """
+
+    current_activity = activity_run(monkeypatch, poll)
+
+    current_activity.poll_for_activity(identity='foo')
+    current_activity.poll.assert_called_with(identity='foo')
+
+
+def test_poll_for_activity_no_identity(monkeypatch, poll=poll):
+    """Test poll_for_activity works without identity passed as param.
+    """
+
+    current_activity = activity_run(monkeypatch, poll)
+
+    current_activity.poll_for_activity()
+    current_activity.poll.assert_called_with(identity=None)
+
+
 def test_run_activity(monkeypatch, poll):
     """Run an activity.
     """
@@ -69,13 +132,25 @@ def test_run_activity(monkeypatch, poll):
     current_activity = activity_run(monkeypatch, poll=poll)
     current_activity.run()
 
-    assert current_activity.poll.called
+    current_activity.poll.assert_called_with(identity=None)
+    assert current_activity.execute_activity.called
+    assert current_activity.complete.called
+
+
+def test_run_activity_identity(monkeypatch, poll):
+    """Run an activity with identity as param.
+    """
+
+    current_activity = activity_run(monkeypatch, poll=poll)
+    current_activity.run(identity='foo')
+
+    current_activity.poll.assert_called_with(identity='foo')
     assert current_activity.execute_activity.called
     assert current_activity.complete.called
 
 
 def test_run_capture_exception(monkeypatch, poll):
-    """Run an activity with an exception raised.
+    """Run an activity with an exception raised during activity execution.
     """
 
     current_activity = activity_run(monkeypatch, poll=poll)
@@ -93,6 +168,48 @@ def test_run_capture_exception(monkeypatch, poll):
     assert not current_activity.complete.called
 
 
+def test_run_capture_fail_exception(monkeypatch, poll):
+    """Run an activity with an exception raised during failing execution.
+    """
+
+    current_activity = activity_run(monkeypatch, poll=poll)
+    current_activity.on_exception = MagicMock()
+    current_activity.execute_activity = MagicMock()
+    current_activity.complete = MagicMock()
+    current_activity.fail = MagicMock()
+    error_msg_long = "Error" * 100
+    current_activity.complete.side_effect = Exception(error_msg_long)
+    current_activity.fail.side_effect = Exception(error_msg_long)
+    current_activity.run()
+
+    assert current_activity.execute_activity.called
+    assert not current_activity.complete.called
+    assert current_activity.fail.called
+    assert current_activity.on_exception.called
+
+
+def test_run_capture_poll_exception(monkeypatch, poll):
+    """Run an activity with an exception raised during poll.
+    """
+
+    current_activity = activity_run(monkeypatch, poll=poll)
+    current_activity.on_exception = MagicMock()
+    current_activity.execute_activity = MagicMock()
+    exception = Exception('poll exception')
+    current_activity.poll.side_effect = exception
+    current_activity.run()
+
+    assert current_activity.poll.called
+    assert current_activity.on_exception.called
+    assert not current_activity.execute_activity.called
+    assert not current_activity.complete.called
+
+    current_activity.on_exception = None
+    current_activity.logger.error = MagicMock()
+    current_activity.run()
+    current_activity.logger.error.assert_called_with(exception, exc_info=True)
+
+
 def test_run_activity_without_id(monkeypatch):
     """Run an activity without an activity id.
     """
@@ -137,8 +254,10 @@ def test_task_failure(monkeypatch, poll):
 
     resp = dict(foo='bar')
     mock = MagicMock(return_value=resp)
+    reason = 'fail'
     current_activity = activity_run(monkeypatch, poll=poll, execute=mock)
-    current_activity.execute_activity.side_effect = Exception('fail')
+    current_activity.on_exception = MagicMock()
+    current_activity.execute_activity.side_effect = Exception(reason)
     current_activity.run()
 
     assert current_activity.fail.called
@@ -151,6 +270,7 @@ def test_task_failure_on_close_activity(monkeypatch, poll):
     resp = dict(foo='bar')
     mock = MagicMock(return_value=resp)
     current_activity = activity_run(monkeypatch, poll=poll, execute=mock)
+    current_activity.on_exception = MagicMock()
     current_activity.execute_activity.side_effect = Exception('fail')
     current_activity.fail.side_effect = Exception('fail')
     current_activity.unset_log_context = MagicMock()
@@ -386,14 +506,17 @@ def test_worker_infinite_loop():
         def __init__(self):
             self.count = 0
 
-        def run(self):
+        def run(self, identity=None):
             spy()
             self.count = self.count + 1
             if self.count < 5:
                 return True
             return False
 
-    activity.worker_runner(Activity())
+    activity_worker = Activity()
+    activity_worker.name = 'activity_name'
+    activity_worker.logger = MagicMock()
+    activity.worker_runner(activity_worker)
     assert spy.called
     assert spy.call_count == 5
 
@@ -592,6 +715,37 @@ def test_create_activity_instance_input_without_decorate(monkeypatch):
     assert resp.get('context') == 'yes'
 
 
+def test_create_activity_instance_input_with_zero_or_empty_values(
+        monkeypatch):
+    """Test the creation of a context for an activity instance input.
+    """
+
+    @task.decorate()
+    def task_a(value1, value2, value3, value4):
+        pass
+
+    activity_mock = MagicMock()
+    activity_mock.name = 'activity'
+    activity_mock.runner = runner.BaseRunner(
+        task_a.fill(
+            value1='zero',
+            value2='empty_list',
+            value3='empty_dict',
+            value4='none'))
+    instance = activity.ActivityInstance(
+        activity_mock,
+        local_context=dict(
+            zero=0, empty_list=[], empty_dict={}, none=None))
+
+    resp = instance.create_execution_input()
+
+    assert len(resp) == 6
+    assert resp.get('zero') == 0
+    assert resp.get('empty_list') == []
+    assert resp.get('empty_dict') == {}
+    assert 'none' not in resp
+
+
 def test_activity_state():
     """Test the creation of the activity state.
     """
