@@ -1,10 +1,7 @@
-from __future__ import absolute_import
-try:
-    from unittest.mock import MagicMock
-except:
-    from mock import MagicMock
-import boto.swf.layer2 as swf
+from unittest.mock import MagicMock
 import json
+
+import boto.swf.layer2 as swf
 import pytest
 
 from garcon import decider
@@ -108,6 +105,75 @@ def test_running_workflow(monkeypatch):
     assert spy.called
 
 
+def test_running_workflow_identity(monkeypatch):
+    """Test running a workflow with and without identity.
+    """
+
+    mock(monkeypatch)
+    from tests.fixtures.flows import example
+
+    d = decider.DeciderWorker(example)
+    d.poll = MagicMock()
+    d.complete = MagicMock()
+    d.create_decisions_from_flow = MagicMock()
+
+    # assert running decider without identity
+    d.run()
+    d.poll.assert_called_with(identity=None)
+
+    # assert running decider with identity
+    d.run('foo')
+    d.poll.assert_called_with(identity='foo')
+
+
+def test_running_workflow_exception(monkeypatch):
+    """Run a decider with an exception raised during poll.
+    """
+
+    mock(monkeypatch)
+    from tests.fixtures.flows import example
+
+    d = decider.DeciderWorker(example)
+    d.poll = MagicMock(return_value=decider_events.history)
+    d.complete = MagicMock()
+    d.create_decisions_from_flow = MagicMock()
+    exception = Exception('test')
+    d.poll.side_effect = exception
+    d.on_exception = MagicMock()
+    d.logger.error = MagicMock()
+    d.run()
+    assert d.on_exception.called
+    d.logger.error.assert_called_with(exception, exc_info=True)
+    assert not d.complete.called
+
+
+def test_create_decisions_from_flow_exception(monkeypatch):
+    """Test exception is raised and workflow fails when exception raised.
+    """
+
+    mock(monkeypatch)
+    from tests.fixtures.flows import example
+
+    decider_worker = decider.DeciderWorker(example)
+    decider_worker.logger.error = MagicMock()
+    decider_worker.on_exception = MagicMock()
+
+    exception = Exception('test')
+    monkeypatch.setattr(decider.activity,
+        'find_available_activities', MagicMock(side_effect = exception))
+
+    mock_decisions = MagicMock()
+    mock_activity_states = MagicMock()
+    mock_context = MagicMock()
+    decider_worker.create_decisions_from_flow(
+        mock_decisions, mock_activity_states, mock_context)
+
+    mock_decisions.fail_workflow_execution.assert_called_with(
+        reason=str(exception))
+    assert decider_worker.on_exception.called
+    decider_worker.logger.error.assert_called_with(exception, exc_info=True)
+
+
 def test_running_workflow_without_events(monkeypatch):
     """Test running a workflow without having any events.
     """
